/**
 * OpenGL 3.1 with GLEW and GLUT
 *
 * @author	Norbert Nopper norbert@nopper.tv
 * @version	1.0
 *
 * Homepage: http://nopper.tv
 *
 * Copyright Norbert Nopper
 */

#include "ShaderProgram.h"

#include <stdio.h>

ShaderProgram::ShaderProgram(void) :
m_vertex(0),
m_fragment(0),
m_program(0)
{
}

ShaderProgram::~ShaderProgram(void)
{
	// Delete everything if not already done.
	deleteAll();
}

GLboolean ShaderProgram::build(const char** vertexSource, const char** fragmentSource)
{
	GLint compiled;
    GLint linked;

	if (!vertexSource || !fragmentSource)
	{
		return GL_FALSE;
	}

	// http://www.opengl.org/sdk/docs/man/xhtml/glCreateShader.xml
	m_vertex = glCreateShader(GL_VERTEX_SHADER);

	// http://www.opengl.org/sdk/docs/man/xhtml/glShaderSource.xml
	glShaderSource(m_vertex, 1, (const char**)vertexSource, 0);

	// http://www.opengl.org/sdk/docs/man/xhtml/glCompileShader.xml
	glCompileShader(m_vertex);

	// http://www.opengl.org/sdk/docs/man/xhtml/glGetShader.xml
    glGetShaderiv(m_vertex, GL_COMPILE_STATUS, &compiled);

	if (!compiled)
	{
		GLint logLength, charsWritten;

		// http://www.opengl.org/sdk/docs/man/xhtml/glGetShader.xml
		glGetShaderiv(m_vertex, GL_INFO_LOG_LENGTH, &logLength);

		char* log = new char[logLength];

		// http://www.opengl.org/sdk/docs/man/xhtml/glGetShaderInfoLog.xml
        glGetShaderInfoLog(m_vertex, logLength, &charsWritten, log);

		printf("Vertex shader compile error:\n");
		printf("%s\n", log);

		delete[] log;

		return GL_FALSE;
	}

	// http://www.opengl.org/sdk/docs/man/xhtml/glCreateShader.xml
	m_fragment = glCreateShader(GL_FRAGMENT_SHADER);

	// http://www.opengl.org/sdk/docs/man/xhtml/glShaderSource.xml
	glShaderSource(m_fragment, 1, (const char**)fragmentSource, 0);

	// http://www.opengl.org/sdk/docs/man/xhtml/glCompileShader.xml
	glCompileShader(m_fragment);

	// http://www.opengl.org/sdk/docs/man/xhtml/glGetShader.xml
    glGetShaderiv(m_fragment, GL_COMPILE_STATUS, &compiled);

	if (!compiled)
	{
		GLint logLength, charsWritten;

		// http://www.opengl.org/sdk/docs/man/xhtml/glGetShader.xml
		glGetShaderiv(m_fragment, GL_INFO_LOG_LENGTH, &logLength);

		char* log = new char[logLength];

		// http://www.opengl.org/sdk/docs/man/xhtml/glGetShaderInfoLog.xml
        glGetShaderInfoLog(m_fragment, logLength, &charsWritten, log);

		printf("Fragment shader compile error:\n");
		printf("%s\n", log);

		delete[] log;

		return GL_FALSE;
	}

	// http://www.opengl.org/sdk/docs/man/xhtml/glCreateProgram.xml
	m_program = glCreateProgram();

	// http://www.opengl.org/sdk/docs/man/xhtml/glAttachShader.xml
    glAttachShader(m_program, m_vertex);

	// http://www.opengl.org/sdk/docs/man/xhtml/glAttachShader.xml
    glAttachShader(m_program, m_fragment);

	// http://www.opengl.org/sdk/docs/man/xhtml/glLinkProgram.xml
    glLinkProgram(m_program);

	// http://www.opengl.org/sdk/docs/man/xhtml/glGetProgram.xml
	glGetProgramiv(m_program, GL_LINK_STATUS, &linked);

	if (!linked)
	{
		GLint logLength, charsWritten;

		// http://www.opengl.org/sdk/docs/man/xhtml/glGetProgram.xml
		glGetProgramiv(m_program, GL_INFO_LOG_LENGTH, &logLength);

		char* log = new char[logLength];

		// http://www.opengl.org/sdk/docs/man/xhtml/glGetProgramInfoLog.xml
		glGetProgramInfoLog(m_program, logLength, &charsWritten, log);

		printf("Shader program link error:\n");
		printf("%s\n", log);
		delete[] log;

		return GL_FALSE;
	}

	return GL_TRUE;
}

void ShaderProgram::deleteAll(void)
{
	if (m_program)
	{
		// http://www.opengl.org/sdk/docs/man/xhtml/glDeleteProgram.xml
		glDeleteProgram(m_program);

		m_program = 0;
	}

	if (m_fragment)
	{
		// http://www.opengl.org/sdk/docs/man/xhtml/glDeleteShader.xml
		glDeleteShader(m_fragment);

		m_fragment = 0;
	}

	if (m_vertex)
	{
		// http://www.opengl.org/sdk/docs/man/xhtml/glDeleteShader.xml
		glDeleteShader(m_vertex);

		m_vertex = 0;
	}
}

GLuint ShaderProgram::getProgram(void)
{
	return m_program;
}